package MDELite6;

/**
 * HomePath -- is a class whose methods determine where libpl and libvm files
 * are relative to certain classes.  This is how MDELite6 executables can 
 * reference key prolog-based files and velocity-templates relative to their jar files
 */
public class HomePath {

    /**
     * key method for MDELite implementations.  Using reflection, it determines
     * where this class is w.r.t. a directory (via its jar file).  From there
     * standard directories like libpl, libvm can be accessed. variables in 
     * Constants are updated
     */
    public static void setHomePath() {
        // Step 1: get initial absolute path to class Main
        Class me = HomePath.class;
        String tmp = me.getProtectionDomain().getCodeSource().getLocation().getPath();
        // in NetBeans tmp will look like "/C:/Users/blah/MDELite/build/classes/"
        if (tmp.startsWith("/") && tmp.contains(":")) {
            tmp = tmp.substring(1);
        }
        
        // Step 2: more cleanup -- exactly why this works I no longer remember :-(
        int pos;
        if (tmp.contains("build/classes/")) {
            pos = tmp.lastIndexOf("build/classes/");
        } else {
            pos = tmp.lastIndexOf("/")+1;
        }
        tmp = tmp.substring(0, pos);
        if (!tmp.contains(":")) {
            tmp = "/" + tmp;
        }
        
        // Step 3: now set global values and print homePath if requested
        Constants.homepath = tmp;
        // running in Netbeans, homePath will look like "C:/Users/blah/MDELite" -- this is the playground for creating files
    }

    /**
     * generally used for debugging purposes -- prints the homepath for an installation
     * @param args -- are ignored
     */
    public static void main(String[] args) {
        setHomePath();
        System.out.format("homepath = %s\n",Constants.homepath);
    }
    
}
