package PrologDB;

/**
 * columns are (name, boolean-isQuoted) pairs.
 * Columns are read-only; they cannot be modified
 * @author don
 */
public class Column {

    /**
     * name of column 
     */
    private final String name;
    /**
     *  does the column contain single-quoted values?
     **/
    private final boolean isQuoted;
    
    // ACCESSOR AND INFO
    
    /**
     * Returns string name of column
     * 
     * @return string name of column
     */
    public String getName() {
        return name;
    }

    /**
     * Returns true if column values are single quoted
     * 
     * @return true if column values are single quoted
     */
    public boolean isQuoted() {
        return isQuoted;
    }
    
    // CONSTRUCTOR
    
    /**
     * create a Column with name and indicator of being single-quoted?
     *
     * @param name of column
     * @param isQuoted is its values single-quoted?
     */
    public Column(String name, boolean isQuoted) {
        this.name = name;
        this.isQuoted = isQuoted;
    }

    // UTILITIES

    /**
     * returns true if 'this' column equals the given otherColumn.
     * Column equality means both columns have the same name
     * and same isQuoted property.
     *
     * @param otherColumn that should be equal to 'this' column
     * @return true if 'this' column equals the given otherColumn.
     */
    public boolean equals(Column otherColumn) {
        if (otherColumn == null) {
            return false;  // nothing to compare
        }
        String tempName = otherColumn.getName();
        if (!name.equals(tempName)) {
            return false;  // names do not match
        }
        if (isQuoted != otherColumn.isQuoted()) {
            return false; // quotation property does not match
        }
        return true;  // they match!
    }
    
    // PRINT
        
    /**
     * Returns a String schema specification of this column, its name in/not-in double quotes
     * @return a String schema specification of this column
     */
    @Override
    public String toString() {
        String q = isQuoted ? "\"" : "";
        return q + name + q;
    }
}
