package PrologDB;

import java.util.HashMap;

/**
 * Unique is a useful utility for evaluating the uniqueness of specific columns
 * of a stream of tuples.
 */
public class Unique {
    /**
     * hashmap of (key,id) pairs for tuples
     */
    HashMap<String,String> hs = new HashMap<>();
    
    /**
     * key is name of key column
     */
    String key;
    
    /**
     * id is name of identifier column
     */
    String id;
    
    /**
     * er is the error reporter, which accumulates error messages
     */
    ErrorReport er;
    
    /**
     * create a Unique object for tuples that have idcol as the name of their
     * column identifier, column as the name of a column that should have unique
     * values, and erReport as the ErrorReport object
     * 
     * @param idcol -- name of identifier column
     * @param column -- name of column in which no duplicates should exist
     * @param erReport -- ErrorReport object for accumulating errors
     */
    public Unique(String idcol, String column, ErrorReport erReport) {
        key = column;
        er = erReport;
        id = idcol;
    }
    
    /**
     * register the content of a tuple: get its idcol key, column value,
     * and see if the column value has been seen before; if so, report an error.
     * 
     * @param t -- tuple whose contents are to be registered
     */
    public void add(Tuple t) {
        String col = t.get(key);
        if (hs.containsKey(col)) {
            er.add(errmsg(col,t.get(id),hs.get(col)));
        } else {
            hs.put(col,t.get(id));
        }
    }
    
    /**
     * generate an error string that two tuples (t1id, t2id) share the
     * same value for column key
     * 
     * @param value
     * @param t1id
     * @param t2id
     * @return error string to be given to ErrorReporter
     */
    
    private String errmsg(String value, String t1id, String t2id) {
        return String.format("unique %s constraint violated -- value='%s' is replicated in tuples %s,%s",key,value,t1id,t2id);
    }
}
