package PrologProductions;

import PrologParser.LineToParse;

/**
 * a token is a primitive unit of parsing
 */
public abstract class Token {
    
    /**
     * string to be parsed
     */
    protected LineToParse toBeParsed;
    
    /** 
     * used when defining statements -- a sequence of tokens
     */
    protected Token() { }
    
    /**
     * used when parsing -- an actual line l is needed
     * @param l -- line to parse
     */
    protected Token(LineToParse l) {
        toBeParsed = l;
    }
    
    /**
     * can this token be parsed?  is the next character
     * of toBeParsed consistent with the requested parsing task?
     * @return true if the next character to parse is consistent with requested parsing task
     */
    public abstract boolean canParse();

    /**
     * actually parse toBeParsed given requested parsing task
     */
    public abstract void parse();
    
    /**
     * set line to parse (l)
     * @param l -- the LineToParse object
     */
    public void setLineToParse(LineToParse l) {
       toBeParsed = l;
    }
}
