package YumlClassParser;

import java.io.*;
import java.util.HashMap;
import java.util.LinkedList;
import MDELite6.Constants;
import MDL.YumlClassParser;
import PrologDB.ErrorReport;

/**
 * main engine for YumlClassParser.
 * @see MDL.YumlClassParser as its facade
 */
public class Main {

    final static String gen = "'^'";
    final static String comp = "'++'";
    final static String agg = "'<>'";
    final static String none = "''";
    final static String arrow1 = "'<'";
    final static String arrow2 = "'>'";
    final static String comma = ",";
    final static String commaApost = ",'";
    final static String apostComma = "',";
    final static String equals = " --> ";
    final static String apost = "'";
    // variables
    static int lineCount = 0;
    static String line, original;
    static String name1, end1, role1;
    static String name2, end2, role2;
    static String[] array1, array2;

    // public variables to other classes
    static HashMap<String, klass> klassS;
    static LinkedList<connection> connectionS;
    static int kounter;
    static PrintStream out;
    static ErrorReport er;

    /**
     * @see MDL.YumlClassParser
     * command line -- 2 arguments full-path-to-yuml-file full-path-to-output-file
     * @param args -- same arguments as MDL.YumlClassParser
     */
    public static void main(String... args) {
        int i;
        int lineNum = 0;

        er = new ErrorReport();

        if (args.length != 2) {
            YumlClassParser.marquee();
        }
        if (!args[0].endsWith(".yuml.yuml") || !args[1].endsWith(".ypl.pl")) {
            YumlClassParser.marquee();
        }

        // initialize static variables
        klassS = new HashMap<>();
        connectionS = new LinkedList<>();
        kounter = 0;

        FileReader fr = null;
        try {
            fr = new FileReader(args[0]);
            BufferedReader in = new BufferedReader(fr);
            out = new PrintStream(args[1]);
            while (true) {
                // foreach line
                name1 = end1 = role1 = name2 = end2 = role2 = "";

                original = line = in.readLine();
                lineNum++;
                if (line == null) {
                    break;
                }
                lineCount++;
                line = line.trim();
                if (line.equals("") || line.startsWith("//")) {
                    continue;
                }

                // lines must start with [name1]
                if (!line.startsWith("[")) {
                    er.add(String.format("%d: can't parse line.  skipping '%s'\n", lineCount, line));
                    continue;
                }
                for (i = 1; i < line.length(); i++) {
                    if (line.charAt(i) == ']') {
                        break;
                    }
                }
                name1 = line.substring(1, i);
                line = line.substring(i + 1);

                // so we have now parsed [name1]
                // lets check to see if it has too many substructures and proceed to create the klass object
                array1 = checkNameStructure(name1);
                new klass(array1, lineCount);

                if (line.equals("")) {
                    continue;
                }

                // the line could end now, or continue with an association
                if (line.startsWith("<>")) {
                    end1 = agg;
                    line = line.substring(2);
                } else if (line.startsWith("++")) {
                    end1 = comp;
                    line = line.substring(2);
                } else if (line.startsWith("<")) {
                    end1 = arrow1;
                    line = line.substring(1);
                } else if (line.startsWith("^")) {
                    end1 = gen;
                    line = line.substring(1);
                } else {
                    end1 = none;
                }
                // end1 contains the arrow ending type
                // now everthing from here to '-' is role1
                for (i = 0; i < line.length(); i++) {
                    if (line.charAt(i) == '-') {
                        break;
                    }
                }
                role1 = line.substring(0, i);

                // now everything from here up to the next token is 
                // role2
                line = line.substring(i + 1);
                if (line.startsWith(".-")) {
                    line = line.substring(2);
                    i = i + 2;
                }
                for (i = 0; i < line.length(); i++) {
                    if (line.substring(i).startsWith("[")) {
                        role2 = line.substring(0, i);
                        line = line.substring(i);
                        end2 = none;
                        break;
                    } else if (line.substring(i).startsWith("++")) {
                        role2 = line.substring(0, i);
                        line = line.substring(i + 2);
                        end2 = comp;
                        break;
                    } else if (line.substring(i).startsWith("<>")) {
                        role2 = line.substring(0, i);
                        line = line.substring(i + 2);
                        end2 = agg;
                        break;
                    } else if (line.substring(i).startsWith(">")) {
                        role2 = line.substring(0, i);
                        line = line.substring(i + 1);
                        end2 = arrow2;
                        break;
                    } else if (line.substring(i).startsWith("^")) {
                        role2 = line.substring(0, i);
                        line = line.substring(i + 1);
                        end2 = gen;
                        break;
                    }
                }
                if (!line.startsWith("[")) {
                    er.add(String.format("%d: expecting [name] after right association\n", lineCount));
                    continue;
                }
                line = line.substring(1); // trim off '['
                for (i = 0; i < line.length(); i++) {
                    if (line.charAt(i) == ']') {
                        break;
                    }
                }
                name2 = line.substring(0, i);
                line = line.substring(i + 1).trim();
                if (!line.equals("")) {
                    er.add(String.format("%d: skipping contents '%s'\n", lineCount, line));
                }

                // lets check to see if it has too many substructures, and proceed to create the klass object
                array2 = checkNameStructure(name2);
                new klass(array2, lineCount);
                // and now create a connection
                new connection(name1, role1, end1, name2, role2, end2);
            }
            out.format("dbase(ypl,[yumlClass,yumlInterface,yumlAssociation]).\n\n");
            klass.writeNodeTuples();
            connection.writeEdgeTuples();
        } catch (IOException e) {
            er.add(String.format("error in parsing %s at line %d : %s\n", args[0], lineNum, e.getMessage()));
        }
        try {
            fr.close();
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }
        out.close();
        File o = new File(Constants.conformFile);
        if (o.length() == 0) {
            o.delete();
        }
        er.printReport(System.err);
    }

    private static String[] checkNameStructure(String name) {

        String[] array = name.replace("|", "@").split("@");
        if (array.length > 3) {
            er.add(String.format("%d: too many substructures in %s\n", lineCount, name));
        }
        switch (array.length) {
            case 3:
                break;  // all is OK
            case 2:
                String[] result = {array[0], array[1], ""};
                array = result;
                break;
            case 1:
            default:
                String[] results = {array[0], "", ""};
                array = results;
        }
        return array;
    }

    // for debugging
    private static void printArray(String header, String[] array) {
        System.out.print(header);
        for (String x : array) {
            System.out.print(x + "@");
        }
        System.out.println();
    }
}
