/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package PrologDB;

import RegTest.Utility;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.List;
//import static org.junit.Assert.*;


public class CommonTst {

    void genericSchemaPrint(String localFileName) {
        try {
            File sFile = new File("TestData/Schema/starTrek.schema.pl");
            DBSchema s = DBSchema.readSchema(sFile, System.err);
            // easy way
            s.print(System.out);
            
            // more detailed way
            System.out.format("\n\n=========\n\n");
            System.out.format("database %s has \n",s.getName());
            for (TableSchema t : s.getTableSchemas()) {
                System.out.format("   table %s with columns ", t.getName());
                for (Column c : t.getColumns()) {
                    char quote = c.isQuoted()?'\'':' ';
                    System.out.format("%c%s%c ",quote,c.getName(),quote);
                }
                System.out.format("\n");
            }
            System.out.format("\n");
            for (SubTableSchema st : s.getSubTableSchemas()) {
                TableSchema supr = st.getSuper();
                System.out.format("   table %s has subtables ", supr.getName());
                for (TableSchema chld : st.getSubTableSchemas()) {
                    System.out.format("%s ", chld.getName());
                }
                System.out.format("\n");
            }
            System.out.format("\n\n");  
            
        }
        catch (Exception e) {
            System.err.println(e.getMessage());
            org.junit.Assert.fail();
        }
    }
    
    
//    TupleList test(Table t) {
//        TupleList tlist = new TupleList(t.schema);
//        t.stream().filter((tup) -> (tup.get("A").equals("a") && tup.get("B").equals("b"))).forEach((tup) -> {
//            tlist.add(tup);
//        });
//        return tlist;
//    }

    void genericCopyDBTest(String localFileName) {
        // localfilename <dbname>.<schemaname>.pl
        DB result = null;
        try {
            result = DB.readDataBase("TestData/DB/" + localFileName);
            String newName = result.getName() + "C";
            result.rename(newName);
            String fullName = result.getFullName();
            result.print(new PrintStream(fullName));
            Utility.validate(fullName, "Correct/DBCopy/" + fullName, false);
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
            org.junit.Assert.fail();
        }
    }

    void genericCopySchemaTest(String localFileName) {
        // localfilename <dbname>.ooschema.pl
        if (!localFileName.endsWith(".ooschema.pl")) {
            System.err.format("file %s does not end in '.ooschema.pl",localFileName);
            org.junit.Assert.fail();
            return;
        }
        DBSchema result = null, copy;
        try {
            File schemaFile = new File("TestData/Schema/" + localFileName);
            result = DBSchema.readSchema(schemaFile, System.err);
            result.flatten();
            String flattenedSchema = localFileName.replace(".ooschema.pl",".schema.pl");
            PrintStream ps = new PrintStream("TestData/Schema/"+flattenedSchema);
            result.print(ps);
            ps.println();
            copy = result.copy();
            String newName = copy.getName() + "C";
            copy.rename(newName);
            String fullName = copy.getFullName();
            PrintStream ps2 = new PrintStream("TestData/" + fullName);
            copy.print(ps2);
            ps2.println();
            ps.close();
            ps2.close();
            Utility.validate("TestData/Schema/"+flattenedSchema, "Correct/SchemaCopy/" + flattenedSchema, false);
            Utility.validate("TestData/" + fullName, "Correct/SchemaCopy/" + fullName, false);
        } catch (Exception ex) {
        	System.err.println(ex.getMessage());
            org.junit.Assert.fail();
        }
    }

    void genericSchema2EmptyDB(String localFileName) {
        // localfilename <dbname>.<schemaname>.pl

        try {
            File schemaFile = new File("TestData/Schema/" + localFileName);
            DBSchema dbs = DBSchema.readSchema(schemaFile, System.err);
            DB dbe = new DB(dbs.getName() + "E", dbs);
            String fullName = dbe.getFullName();
            dbe.print(new PrintStream(fullName));
            Utility.validate(fullName, "Correct/EmptyDB/" + fullName, false);
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
            org.junit.Assert.fail();
        }
    }

    protected void genericError(String localSchemaName) {
        String inputFile = "TestData/SchemaErrors/" + localSchemaName;
        String outputFile = "ERR" + localSchemaName;
        String correctFile = "Correct/SchemaErrors/" + "ERR" + localSchemaName;

        try {
            File schemafile = new File(inputFile);
            RegTest.Utility.redirectStdErr(outputFile);
            DBSchema.readSchema(schemafile, System.err);
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }
        System.err.close();
        RegTest.Utility.validate(outputFile, correctFile, false);
    }

    protected void genericExtend(String localdbfile) {
        DB db = DB.readDataBase("TestData/DB/" + localdbfile);
        DBSchema dbs = db.getSchema();
        String name = dbs.getName();
        
        // build schema from ground up
        // RULE: always create schema first before creating database.
        // NOt other way around.
        
        DBSchema newSchema = new DBSchema(name + "1");
        newSchema.flatten();  // because what we are copying is a flattened schema

        for (TableSchema ts : dbs.getTableSchemas()) {
            TableSchema nts = new TableSchema(ts.getName());
            for (Column c : ts.getColumns()) {
                nts.addColumn(new Column(c.getName(),c.isQuoted()));
            }
            nts.addColumn(new Column("length",false)); // this is the extra attribute that defined "extension" in this test
            newSchema.addTableSchema(nts);
        }
        
        // now finish off with subtable declarations
        List<SubTableSchema> slist = db.getSubTableSchemas();
        for (SubTableSchema sts : slist) {
            newSchema.addSubTableSchema(sts.copy(newSchema));
        }
        // now at this point, we have literally copied the contents of dbs into new schema.
        // now we can instantiate tables -- because newSchema can be normalized

        DB db1 = new DB(db.getName() + "1", newSchema); // this creates empty tables
        for (Table t : db.getTables()) {
            Table nt = db1.getTableEH(t.getName());
            TableSchema newts = newSchema.getTableSchemaEH(t.getName());
            for (Tuple tpl : t.tuplesLocal()) {
                Tuple tuple = new Tuple(newts);
                String firstString = null;
                for (Column c : t.getColumns()) {
                    String colName = c.getName();
                    if (c.isQuoted() && firstString == null) {
                        firstString = tpl.get(colName);
                        String firstValue = firstString.length() + "";
                        tuple.addColumnValue("length", firstValue);
                    }
                    tuple.addColumnValue(colName, tpl.get(colName));
                }
                tuple.isComplete();
                nt.add(tuple);
            }
        }

        String db1filename = db1.getFullName();
        try {
            db1.print(new PrintStream(db1filename));
        } catch (FileNotFoundException ex) {
            throw new RuntimeException(ex.getMessage());
        }
        Utility.validate(db1filename, "Correct/DBExtend/" + db1filename, false);
    }

}
