package PrologDB;

import java.io.File;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.junit.Test;

public class DocExamplesTest extends CommonTst {

    public DocExamplesTest() {
    }

    @Test
    public void testprintSchema() throws Exception {
        RegTest.Utility.redirectStdOut("printSchema.txt");
        printSchema();
        RegTest.Utility.validate("printSchema.txt", "Correct/DB/printSchema.txt", false);
    }

///printSchema
    void printSchema() throws Exception {
        System.out.format("START print Schema\n\n");
        File sFile = new File("TestData/Schema/starTrek.ooschema.pl");
        DBSchema s = DBSchema.readSchema(sFile, System.err);
        s.flatten();

        // easy way
        s.print(System.out);

        // more detailed way
        System.out.format("\n=========\n\n");
        System.out.format("database %s has \n", s.getName());
        for (TableSchema t : s.getTableSchemas()) {
            System.out.format("   table %10s with columns ", t.getName());
            for (Column c : t.getColumns()) {
                String quote = c.isQuoted() ? "\'" : "";
                System.out.format("%s%s%s ", quote, c.getName(), quote);
            }
            System.out.format("\n");
        }
        System.out.format("\n");
        for (SubTableSchema st : s.getSubTableSchemas()) {
            TableSchema supr = st.getSuper();
            System.out.format("   table %10s has subtables ", supr.getName());
            for (TableSchema chld : st.getSubTableSchemas()) {
                System.out.format("%s ", chld.getName());
            }
            System.out.format("\n");
        }
        System.out.format("\n\nEND print Schema\n");
    }
///printSchema

///schemaBuild
    DBSchema schemaBuild() throws Exception {
        DBSchema trekSchema = new DBSchema("starTrek");

        //  create tables for ooschema
        TableSchema crewman = new TableSchema("crewman");
        crewman.addColumns(
                new Column("cid", false), // false means non-quoted
                new Column("fname", false),
                new Column("lname", false));
        
        TableSchema commander = new TableSchema("commander");
        commander.addColumn(new Column("rank", false));

        TableSchema lieutenant = new TableSchema("lieutenant");
        lieutenant.addColumn(new Column("specialty", false));

        trekSchema.addTableSchema(crewman);
        trekSchema.addTableSchema(commander);
        trekSchema.addTableSchema(lieutenant);

        //  now createe the lone subtable declaration, and add to schema
        SubTableSchema sts = new SubTableSchema(crewman);
        sts.addSubTableSchema(commander);
        sts.addSubTableSchema(lieutenant);
        trekSchema.addSubTableSchema(sts);

        // now print the schema declaration
        return trekSchema;
    }
///schemaBuild

    @Test
    public void testSchemaBuild() throws Exception {
        RegTest.Utility.redirectStdOut("schemaBuild.txt");
        System.out.format("\n\n START schema build \n\n");
        DBSchema trekSchema = schemaBuild();
        trekSchema.print(System.out);
        System.out.format("\n\n END schema build \n\n");
        RegTest.Utility.validate("schemaBuild.txt", "Correct/DB/schemaBuild.txt", false);
    }

    @Test
    public void testDBread() throws Exception {
        RegTest.Utility.redirectStdOut("dbread.txt");
        DBread();
        RegTest.Utility.validate("dbread.txt", "Correct/DB/dbread.txt", false);
    }

///DBread
    void DBread() throws Exception {
        System.out.format("\n\n START database read \n\n");
        File dbfile = new File("TestData/DB/enterprise.starTrek.pl");
        DB db = DB.readDataBase(dbfile, System.err);
        
        // easy way to print
        db.print(System.out);

        // more detailed way
        System.out.format("\n\n ===== \n\n");
        System.out.format("database %s contains:\n", db.getName());

        for (Table t : db.getTables()) {
            System.out.format("Table %s\n", t.getName());
            for (Tuple tup : t.tuples()) {
                tup.print(System.out);
            }
            System.out.format("\n");
        }
        System.out.format("\n\n END database read \n\n");
    }
///DBread

    @Test
    public void testDBbuild() throws Exception {
        RegTest.Utility.redirectStdOut("dbBuild.txt");
        DBBuild();
        RegTest.Utility.validate("dbBuild.txt", "Correct/DB/dbBuild.txt", false);
    }

///DBBuild
    void DBBuild() throws Exception {
        System.out.format("\n\n\n START build enterprise database\n\n");
        DBSchema trekSchema = schemaBuild();
        trekSchema.flatten();
        DB enterprise = new DB("enterprise", trekSchema);

        // create spock tuple
        Table crewman = enterprise.getTableEH("crewman");
        Tuple spock = new Tuple(crewman);
        spock.addColumnValuesEH("c1", "mr", "spock");
        crewman.add(spock);

        //  create sulu tuple
        Table lieutenant = enterprise.getTableEH("lieutenant");
        Tuple sulu = new Tuple(lieutenant);
        sulu.addColumnValuesEH("c3", "hikaru", "sulo", "navigation");
        lieutenant.add(sulu);

        //  create kirk tupld
        Table commander = enterprise.getTableEH("commander");
        Tuple kirk = new Tuple(commander);
        kirk.addColumnValuesEH("c2", "james", "kirk", "captain");
        commander.add(kirk);

        //  now print database
        enterprise.print(System.out);
        System.out.format("\n\n END build enterprise database \n\n");
    }
///DBBuild

    @Test
    public void testTableRetrieve() {
        RegTest.Utility.redirectStdOut("tblRetrieve.txt");
        TableRetrieve();
        RegTest.Utility.validate("tblRetrieve.txt", "Correct/DB/tblRetrieve.txt", false);
    }

///TableRetrieve
    void TableRetrieve() {
        System.out.format("BEGIN Table Retrieval\n\n");
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");
        Table dog = db.getTable("dog");

        // query 1: retrieve all aussies
        dog.stream().filter(t -> t.is("breed", "aussie")).forEach(t -> t.print(System.out));
        System.out.println();

        // query 1': retrieve all aussies -- version 2
        dog.filter(t -> t.is("breed", "aussie")).print(System.out);

        // query 2: retrieve all aussies that are blacktri's
        Stream<Tuple> aussies = dog.stream().filter(t -> t.is("breed", "aussie"));
        Stream<Tuple> cuteAussies = aussies.filter(t -> t.is("color", "blacktri"));
        cuteAussies.forEach(t -> t.print(System.out));
        System.out.println();

        // query 2: another version, this time using TupleLists
        dog.filter(t -> t.is("breed", "aussie") && t.is("color", "blacktri"))
                .print(System.out);
        System.out.format("END Table Retrieval");
    }
///TableRetrieve

    @Test
    public void InheritanceRetrieval() {
        RegTest.Utility.redirectStdOut("inhRetrieve.txt");
        InheritanceRetrieve();
        RegTest.Utility.validate("inhRetrieve.txt", "Correct/DB/inhRetrieve.txt", false);
    }

///InheritanceRetrieve
    void InheritanceRetrieve() {
        System.out.format("BEGIN DBTable Retrieval\n\n");
        DB db = DB.readDataBase("TestData/DB/pets.petdb.pl");
        Table pet = db.getTableEH("pet");

        // query 1: retrieve all pets
        pet.stream().forEach(t -> t.print(System.out));

        // query 2: all pets whose name begins with "l" -- first way
        System.out.println("\nlist of all pets whose names start with 'l'");
        pet.stream().filter(t -> t.get("name").startsWith("l")).forEach(t -> t.print(System.out));

        // query 2': all pets whose name begins with "l" -- second way
        System.out.println("\nversion 2");
        Predicate<Tuple> p = t -> t.get("name").startsWith("l");
        pet.stream().filter(p).forEach(t -> t.print(System.out));

        // query 2'': all pets whose name begins with "l" -- third way
        System.out.println("\nversion 3");
        Predicate<Tuple> p1 = t -> t.get("name").startsWith("l");
        pet.filter(p1).project("name").print(System.out);

        System.out.format("END DBTable Retrieval\n\n");
    }
///InheritanceRetrieve

    @Test
    public void crossSchemaTest() {
        RegTest.Utility.redirectStdOut("cross.txt");
        System.out.format("BEGIN cross schema tests \n\n");
        crossSchema();
        System.out.format("\nEND cross schema tests");

        RegTest.Utility.validate("cross.txt", "Correct/DB/cross.txt", false);
    }

///crossSchema
    public void crossSchema() {
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");

        // Step 1: cross the dog and owner schemas
        TableSchema dogSchema = db.getTableSchema("dog");
        dogSchema.print(System.out);
        TableSchema ownerSchema = db.getTableSchema("owner");
        ownerSchema.print(System.out);
        TableSchema dXoSchema = dogSchema.crossProduct(ownerSchema);
        dXoSchema.print(System.out);

        // Step 2: get the tables to cross
        Table dog = db.getTable("dog");
        Table when = db.getTable("when");
        Table owner = db.getTable("owner");

        // Step 3: take cross product of schemas
        when.schema.print(System.out);
        TableSchema dogXwhen = dog.schema.crossProduct(when.schema);
        dogXwhen.print(System.out);
        TableSchema dogXwhenXowner = dogXwhen.crossProduct(owner.schema);
        dogXwhenXowner.print(System.out);
    }
///crossSchema

    @Test
    public void JoinTest() {
        RegTest.Utility.redirectStdOut("join.txt");
        TableJoins();
        RegTest.Utility.validate("join.txt", "Correct/DB/join.txt", false);
    }

///TableJoins
    void TableJoins() {
        System.out.format("BEGIN cross schema tests \n\n");
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");

        // Step 1: get the tables to cross
        Table dog = db.getTable("dog");
        Table when = db.getTable("when");
        Table owner = db.getTable("owner");

        // Step 2: join tables
        Table dogXwhen = Table.join(dog, "did", when, "did");
        dogXwhen.print(System.out);
        Table dogXwhenXowner = Table.join(dogXwhen, "when.oid", owner, "oid");
        dogXwhenXowner.print(System.out);
    }
///TableJoins

    @Test
    public void selfJoinTest() {
        RegTest.Utility.redirectStdOut("join2.txt");
        selfJoin();
        RegTest.Utility.validate("join2.txt", "Correct/DB/join2.txt", false);
    }

///SelfJoins
    void selfJoin() {
        System.out.format("BEGIN cross schema tests \n\n");
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");

        // Step 1: get the tables to cross
        Table dog = db.getTable("dog");
        Table dog2 = dog.copyForSelfJoins("dog2");

        // Step 2: join tables
        Table dogXdog2 = Table.join(dog, "did", dog2, "did");
        dogXdog2.print(System.out);
    }
///SelfJoins

    @Test
    public void uniqueTest() {
        RegTest.Utility.redirectStdOut("unique.txt");
        Unique();
        RegTest.Utility.validate("unique.txt", "Correct/DB/unique.txt", false);
    }

///Unique
    void Unique() {
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");
        Table dog = db.getTable("dog");
        ErrorReport er = new ErrorReport();
        Unique u = new Unique("did", "name", er);
        dog.stream().forEach(t -> u.add(t));
        try {
            er.printReport(System.out);
        } catch (Exception e) {
        }
    }
///Unique

    @Test
    public void filterTest() {
        RegTest.Utility.redirectStdOut("ftest.txt");
        System.out.format("START filterTest \n\n");
        filter();
        System.out.format("\nEND filterTest");
        RegTest.Utility.validate("ftest.txt", "Correct/DB/ftest.txt", false);
    }

///filter
    public void filter() {
        try {
            File dbfile = new File("TestData/DB/enterprise.starTrek.pl");
            DB db = DB.readDataBase(dbfile, System.err);

            System.out.println("first way");
            Table crew = db.getTableEH("crewman");
            crew.schema.print(System.out);
            for (Tuple t : crew.tuples()) {
                if (t.get("fname").startsWith("h")) {
                    t.print(System.out);
                }
            }

            System.out.println("\n\nsecond way");
            crew.schema.print(System.out);
            crew.stream()
                    .filter(t -> t.get("fname").startsWith("h"))
                    .forEach(t -> t.print(System.out));

            System.out.println("\n\nthird way");
            Predicate<Tuple> p = t -> t.get("fname").startsWith("h");
            crew.filter(p).print(System.out);
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }
///filter

    @Test
    public void conversion() {
        RegTest.Utility.redirectStdOut("cnv.txt");
        DB db = DB.readDataBase("TestData/DB/enterprise.starTrek.pl");
        Table crew = db.getTableEH("crewman");

        Stream<Tuple> crewStream = crew.stream();

        List<Tuple> crewList = crewStream.collect(Collectors.toList());

        for (Tuple t : crewList) {
            t.print(System.out);
        }

        crewList.stream().forEach(t -> t.print(System.out));
        RegTest.Utility.validate("cnv.txt", "Correct/DB/cnv.txt", false);
    }

    @Test
    public void conformTest() {
        RegTest.Utility.redirectStdOut("cnf.txt");
        conform();
        RegTest.Utility.validate("cnf.txt", "Correct/DB/cnf.txt", false);
    }

///conform
    public void conform() {
        DB db = DB.readDataBase("TestData/DB/ex.con.pl");

        Table contract = db.getTableEH("contract");

        ErrorReport er = new ErrorReport();

        // Botn Constraint
        contract.stream()
                .filter(t -> ((t.isNull("pid") && t.isNull("cid"))
                        || (!t.isNull("pid") && !t.isNull("cid"))))
                .forEach(t -> er.add("Both constraint violated " + t.get("kid")));

        // Company Constraint
        contract.stream()
                .filter(t -> (!t.isNull("cid") && t.getInt("value") < 500))
                .forEach(t -> er.add("company constraint violated " + t.get("kid")));

        // Person Constraint
        contract.stream()
                .filter(t -> !t.isNull("pid") && t.getInt("value") > 100)
                .forEach(t -> er.add("person constraint violated " + t.get("kid")));

        try {
            er.printReport(System.out);
        } catch (Exception e) {
        }
    }
///conform

    @Test
    public void nestedLoop() {
        RegTest.Utility.redirectStdOut("nl.txt");
        DB db = DB.readDataBase("TestData/DB/dogOwner.do.pl");

        Table dog = db.getTableEH("dog");
        ErrorReport er = new ErrorReport();

        dog.stream().filter(t -> dog.stream().filter(x -> x.get("name").equals(t.get("name"))).count() > 1)
                .forEach(t -> er.add("name duplicate " + t.get("name")));

        try {
            er.printReport(System.out);
        } catch (RuntimeException e) {
            System.out.println("errors found");
        }
        System.out.println("DONE");
        RegTest.Utility.validate("nl.txt", "Correct/DB/nl.txt", false);
    }
    
    
///csvfile
    public void csvfile() {
        Table csv = Table.readTable("TestData/CSV/MixinPass6.csv");
        csv.writeTable("NormalProlog.pl");
        csv.writeTable("AsCSVFile.csv");
    }
///csvfile

}
